@@ -1,5 +1,6 @@
 import collections
 
+from .constants import DEFAULT_LINK_FIELDS
 from .graphql import FIELD_VALUE, GraphQlQuery
 
 
@@ -30,16 +31,7 @@ def add_links_fields(entity_field, nested_fields):
         return
     links_fields = nested_fields.pop("links")
 
-    link_edge_fields = {
-        "id",
-        "linkType",
-        "projectName",
-        "entityType",
-        "entityId",
-        "direction",
-        "description",
-        "author",
-    }
+    link_edge_fields = set(DEFAULT_LINK_FIELDS)
     if isinstance(links_fields, dict):
         simple_fields = set(links_fields)
         simple_variant = len(simple_fields - link_edge_fields) == 0
@@ -49,8 +41,12 @@ def add_links_fields(entity_field, nested_fields):
 
     link_field = entity_field.add_field_with_edges("links")
 
+    link_name_var = link_field.add_variable("linkNames", "[String!]")
+    link_name_regex_var = link_field.add_variable("linkNameRegex", "String!")
     link_type_var = link_field.add_variable("linkTypes", "[String!]")
     link_dir_var = link_field.add_variable("linkDirection", "String!")
+    link_field.set_filter("names", link_name_var)
+    link_field.set_filter("nameEx", link_name_regex_var)
     link_field.set_filter("linkTypes", link_type_var)
     link_field.set_filter("direction", link_dir_var)
 
@@ -175,10 +171,15 @@ def folders_graphql_query(fields):
     folder_ids_var = query.add_variable("folderIds", "[String!]")
     parent_folder_ids_var = query.add_variable("parentFolderIds", "[String!]")
     folder_paths_var = query.add_variable("folderPaths", "[String!]")
+    folder_path_regex_var = query.add_variable("folderPathRegex", "String!")
     folder_names_var = query.add_variable("folderNames", "[String!]")
     folder_types_var = query.add_variable("folderTypes", "[String!]")
-    statuses_var = query.add_variable("folderStatuses", "[String!]")
     has_products_var = query.add_variable("folderHasProducts", "Boolean!")
+    has_tasks_var = query.add_variable("folderHasTasks", "Boolean!")
+    has_links_var = query.add_variable("folderHasLinks", "HasLinksFilter")
+    has_children_var = query.add_variable("folderHasChildren", "Boolean!")
+    statuses_var = query.add_variable("folderStatuses", "[String!]")
+    tags_var = query.add_variable("folderTags", "[String!]")
 
     project_field = query.add_field("project")
     project_field.set_filter("name", project_name_var)
@@ -188,9 +189,14 @@ def folders_graphql_query(fields):
     folders_field.set_filter("parentIds", parent_folder_ids_var)
     folders_field.set_filter("names", folder_names_var)
     folders_field.set_filter("paths", folder_paths_var)
+    folders_field.set_filter("pathEx", folder_path_regex_var)
     folders_field.set_filter("folderTypes", folder_types_var)
     folders_field.set_filter("statuses", statuses_var)
+    folders_field.set_filter("tags", tags_var)
     folders_field.set_filter("hasProducts", has_products_var)
+    folders_field.set_filter("hasTasks", has_tasks_var)
+    folders_field.set_filter("hasLinks", has_links_var)
+    folders_field.set_filter("hasChildren", has_children_var)
 
     nested_fields = fields_to_dict(fields)
     add_links_fields(folders_field, nested_fields)
@@ -218,6 +224,10 @@ def tasks_graphql_query(fields):
     task_names_var = query.add_variable("taskNames", "[String!]")
     task_types_var = query.add_variable("taskTypes", "[String!]")
     folder_ids_var = query.add_variable("folderIds", "[String!]")
+    assignees_any_var = query.add_variable("taskAssigneesAny", "[String!]")
+    assignees_all_var = query.add_variable("taskAssigneesAll", "[String!]")
+    statuses_var = query.add_variable("taskStatuses", "[String!]")
+    tags_var = query.add_variable("taskTags", "[String!]")
 
     project_field = query.add_field("project")
     project_field.set_filter("name", project_name_var)
@@ -228,6 +238,56 @@ def tasks_graphql_query(fields):
     tasks_field.set_filter("names", task_names_var)
     tasks_field.set_filter("taskTypes", task_types_var)
     tasks_field.set_filter("folderIds", folder_ids_var)
+    tasks_field.set_filter("assigneesAny", assignees_any_var)
+    tasks_field.set_filter("assignees", assignees_all_var)
+    tasks_field.set_filter("statuses", statuses_var)
+    tasks_field.set_filter("tags", tags_var)
+
+    nested_fields = fields_to_dict(fields)
+    add_links_fields(tasks_field, nested_fields)
+
+    query_queue = collections.deque()
+    for key, value in nested_fields.items():
+        query_queue.append((key, value, tasks_field))
+
+    while query_queue:
+        item = query_queue.popleft()
+        key, value, parent = item
+        field = parent.add_field(key)
+        if value is FIELD_VALUE:
+            continue
+
+        for k, v in value.items():
+            query_queue.append((k, v, field))
+    return query
+
+
+def tasks_by_folder_paths_graphql_query(fields):
+    query = GraphQlQuery("TasksByFolderPathQuery")
+    project_name_var = query.add_variable("projectName", "String!")
+    task_names_var = query.add_variable("taskNames", "[String!]")
+    task_types_var = query.add_variable("taskTypes", "[String!]")
+    folder_paths_var = query.add_variable("folderPaths", "[String!]")
+    assignees_any_var = query.add_variable("taskAssigneesAny", "[String!]")
+    assignees_all_var = query.add_variable("taskAssigneesAll", "[String!]")
+    statuses_var = query.add_variable("taskStatuses", "[String!]")
+    tags_var = query.add_variable("taskTags", "[String!]")
+
+    project_field = query.add_field("project")
+    project_field.set_filter("name", project_name_var)
+
+    folders_field = project_field.add_field_with_edges("folders")
+    folders_field.add_field("path")
+    folders_field.set_filter("paths", folder_paths_var)
+
+    tasks_field = folders_field.add_field_with_edges("tasks")
+    # WARNING: At moment when this been created 'names' filter is not supported
+    tasks_field.set_filter("names", task_names_var)
+    tasks_field.set_filter("taskTypes", task_types_var)
+    tasks_field.set_filter("assigneesAny", assignees_any_var)
+    tasks_field.set_filter("assignees", assignees_all_var)
+    tasks_field.set_filter("statuses", statuses_var)
+    tasks_field.set_filter("tags", tags_var)
 
     nested_fields = fields_to_dict(fields)
     add_links_fields(tasks_field, nested_fields)
@@ -256,7 +316,10 @@ def products_graphql_query(fields):
     product_names_var = query.add_variable("productNames", "[String!]")
     folder_ids_var = query.add_variable("folderIds", "[String!]")
     product_types_var = query.add_variable("productTypes", "[String!]")
-    statuses_var = query.add_variable("statuses", "[String!]")
+    product_name_regex_var = query.add_variable("productNameRegex", "String!")
+    product_path_regex_var = query.add_variable("productPathRegex", "String!")
+    statuses_var = query.add_variable("productStatuses.", "[String!]")
+    tags_var = query.add_variable("productTags.", "[String!]")
 
     project_field = query.add_field("project")
     project_field.set_filter("name", project_name_var)
@@ -267,6 +330,9 @@ def products_graphql_query(fields):
     products_field.set_filter("folderIds", folder_ids_var)
     products_field.set_filter("productTypes", product_types_var)
     products_field.set_filter("statuses", statuses_var)
+    products_field.set_filter("tags", tags_var)
+    products_field.set_filter("nameEx", product_name_regex_var)
+    products_field.set_filter("pathEx", product_path_regex_var)
 
     nested_fields = fields_to_dict(set(fields))
     add_links_fields(products_field, nested_fields)
@@ -299,24 +365,28 @@ def versions_graphql_query(fields):
     hero_or_latest_only_var = query.add_variable(
         "heroOrLatestOnly", "Boolean"
     )
+    statuses_var = query.add_variable("versionStatuses", "[String!]")
+    tags_var = query.add_variable("versionTags", "[String!]")
 
     project_field = query.add_field("project")
     project_field.set_filter("name", project_name_var)
 
-    products_field = project_field.add_field_with_edges("versions")
-    products_field.set_filter("ids", version_ids_var)
-    products_field.set_filter("productIds", product_ids_var)
-    products_field.set_filter("versions", versions_var)
-    products_field.set_filter("heroOnly", hero_only_var)
-    products_field.set_filter("latestOnly", latest_only_var)
-    products_field.set_filter("heroOrLatestOnly", hero_or_latest_only_var)
+    versions_field = project_field.add_field_with_edges("versions")
+    versions_field.set_filter("ids", version_ids_var)
+    versions_field.set_filter("productIds", product_ids_var)
+    versions_field.set_filter("versions", versions_var)
+    versions_field.set_filter("heroOnly", hero_only_var)
+    versions_field.set_filter("latestOnly", latest_only_var)
+    versions_field.set_filter("heroOrLatestOnly", hero_or_latest_only_var)
+    versions_field.set_filter("statuses", statuses_var)
+    versions_field.set_filter("tags", tags_var)
 
     nested_fields = fields_to_dict(set(fields))
-    add_links_fields(products_field, nested_fields)
+    add_links_fields(versions_field, nested_fields)
 
     query_queue = collections.deque()
     for key, value in nested_fields.items():
-        query_queue.append((key, value, products_field))
+        query_queue.append((key, value, versions_field))
 
     while query_queue:
         item = query_queue.popleft()
@@ -337,6 +407,13 @@ def representations_graphql_query(fields):
     repre_ids_var = query.add_variable("representationIds", "[String!]")
     repre_names_var = query.add_variable("representationNames", "[String!]")
     version_ids_var = query.add_variable("versionIds", "[String!]")
+    has_links_var = query.add_variable("representationHasLinks", "HasLinksFilter")
+    statuses_var = query.add_variable(
+        "representationStatuses", "[String!]"
+    )
+    tags_var = query.add_variable(
+        "representationTags", "[String!]"
+    )
 
     project_field = query.add_field("project")
     project_field.set_filter("name", project_name_var)
@@ -345,6 +422,9 @@ def representations_graphql_query(fields):
     repres_field.set_filter("ids", repre_ids_var)
     repres_field.set_filter("versionIds", version_ids_var)
     repres_field.set_filter("names", repre_names_var)
+    repres_field.set_filter("hasLinks", has_links_var)
+    repres_field.set_filter("statuses", statuses_var)
+    repres_field.set_filter("tags", tags_var)
 
     nested_fields = fields_to_dict(set(fields))
     add_links_fields(repres_field, nested_fields)
@@ -412,6 +492,10 @@ def workfiles_info_graphql_query(fields):
     workfiles_info_ids = query.add_variable("workfileIds", "[String!]")
     task_ids_var = query.add_variable("taskIds", "[String!]")
     paths_var = query.add_variable("paths", "[String!]")
+    path_regex_var = query.add_variable("workfilePathRegex", "String!")
+    has_links_var = query.add_variable("workfilehasLinks", "HasLinksFilter")
+    statuses_var = query.add_variable("workfileStatuses", "[String!]")
+    tags_var = query.add_variable("workfileTags", "[String!]")
 
     project_field = query.add_field("project")
     project_field.set_filter("name", project_name_var)
@@ -420,6 +504,10 @@ def workfiles_info_graphql_query(fields):
     workfiles_field.set_filter("ids", workfiles_info_ids)
     workfiles_field.set_filter("taskIds", task_ids_var)
     workfiles_field.set_filter("paths", paths_var)
+    workfiles_field.set_filter("pathEx", path_regex_var)
+    workfiles_field.set_filter("hasLinks", has_links_var)
+    workfiles_field.set_filter("statuses", statuses_var)
+    workfiles_field.set_filter("tags", tags_var)
 
     nested_fields = fields_to_dict(set(fields))
     add_links_fields(workfiles_field, nested_fields)
@@ -447,6 +535,9 @@ def events_graphql_query(fields):
     states_var = query.add_variable("eventStates", "[String!]")
     users_var = query.add_variable("eventUsers", "[String!]")
     include_logs_var = query.add_variable("includeLogsFilter", "Boolean!")
+    has_children_var = query.add_variable("hasChildrenFilter", "Boolean!")
+    newer_than_var = query.add_variable("newerThanFilter", "String!")
+    older_than_var = query.add_variable("olderThanFilter", "String!")
 
     events_field = query.add_field_with_edges("events")
     events_field.set_filter("topics", topics_var)
@@ -454,6 +545,9 @@ def events_graphql_query(fields):
     events_field.set_filter("states", states_var)
     events_field.set_filter("users", users_var)
     events_field.set_filter("includeLogs", include_logs_var)
+    events_field.set_filter("hasChildren", has_children_var)
+    events_field.set_filter("newerThan", newer_than_var)
+    events_field.set_filter("olderThan", older_than_var)
 
     nested_fields = fields_to_dict(set(fields))
 
